@@ -6,7 +6,7 @@ use integer;
 use base qw(Exporter);
 use Carp;
 
-our $VERSION   = '1.12';
+our $VERSION   = '1.14';
 our @EXPORT_OK = qw(grep_set map_set grep_spans map_spans);
 
 use overload
@@ -841,54 +841,29 @@ sub member
 {
     my($set, $n) = @_;
 
-    my $inSet = $set->{negInf};
-    my $edge  = $set->{edges};
+    my $i = _bsearch($set->{edges}, $n);
 
-    for (my $i=0; $i<@$edge; $i++)
-    {
-	if ($inSet)
-	{
-	    return 1 if $n <= $$edge[$i];
-	    $inSet = 0;
-	}
-	else
-	{
-	    return 0 if $n <= $$edge[$i];
-	    $inSet = 1;
-	}
-    }
-
-    $inSet
+    $set->{negInf} xor $i & 1
 }
 
+use constant INSERT => 0;
+use constant REMOVE => 1;
 
-sub insert
+sub insert { _indel(@_, INSERT); }
+sub remove { _indel(@_, REMOVE); }
+
+sub _indel # INsertion/DELetion
 {
-    my($set, $n) = @_;
+    my($set, $n, $indel) = @_;
     defined $n or return;
 
-    my $inSet = $set->{negInf};
     my $edge = $set->{edges};
+    my $i    = _bsearch($edge, $n);
 
-    my $i;
-    for ($i=0; $i<@$edge; $i++)
-    {
-	if ($inSet)
-	{
-	    $n <= $edge->[$i] and return;
-	    $inSet = 0;
-	}
-	else
-	{
-	    $n <= $edge->[$i] and last;
-	    $inSet = 1;
-	}
-    }
+    return if $set->{negInf} xor $i & 1 xor $indel;
 
-    $inSet and return;
-
-    my $lGap = $i==0      || $n-1 - $edge->[$i-1];
-    my $rGap = $i==@$edge || $edge->[$i] - $n;
+    my $lGap = $i==0      || $edge->[$i-1] < $n-1;
+    my $rGap = $i==@$edge || $n            < $edge->[$i];
 
     if    (    $lGap and     $rGap) { splice @$edge, $i, 0, $n-1, $n }
     elsif (not $lGap and     $rGap) { $edge->[$i-1]++                }
@@ -896,34 +871,35 @@ sub insert
     else                            { splice @$edge, $i-1, 2         }
 }
 
-
-sub remove
+# Returns the index of the first edge that satisifies target <= edge.
+# Returns $#$edges+1 if target > the last edge.
+# Returns 0 if edges is empty.
+sub _bsearch
 {
-    my($set, $n) = @_;
-    defined $n or return;
+    my($edges, $target) = @_;
 
-    my $inSet = $set->{negInf};
-    my $edge  = $set->{edges};
+    @$edges or return 0;
 
-    my $i;
-    for ($i=0; $i<@$edge; $i++)
+    my $lower = 0;
+    my $upper = $#$edges;
+
+    while ($lower+1 < $upper)
     {
-	if ($inSet)
+	my $mid = int(($lower + $upper) / 2);
+
+	if ($target <= $edges->[$mid])
 	{
-	    last if $n <= $$edge[$i];
-	    $inSet = 0;
+	    $upper = $mid;
 	}
 	else
 	{
-	    return if $n <= $$edge[$i];
-	    $inSet = 1;
+	    $lower = $mid+1;
 	}
     }
 
-    return unless $inSet;
-
-    splice @{$set->{edges}}, $i, 0, $n-1, $n;
-    $set->_cleanup;
+    $target <= $edges->[$lower] and return $lower;
+    $target <= $edges->[$upper] and return $upper;
+    $upper + 1
 }
 
 
@@ -1358,7 +1334,6 @@ sub at
     $i < 0 ? $set->_at_neg($i) : $set->_at_pos($i)
 }
 
-
 sub _at_pos
 {
     my($set, $i) = @_;
@@ -1406,6 +1381,38 @@ sub _at_neg
     @edges ? $edges[0] + $i : undef
 }
 
+sub ord
+{
+    my($set, $n) = @_;
+
+    $set->{negInf} and
+	croak "Set::IntSpan::ord: negative infinite set\n";
+
+    defined $n or return undef;
+
+    my $i = 0;
+    my @edges = @{$set->{edges}};
+
+    while (@edges)
+    {
+	my($lower, $upper) = splice(@edges, 0, 2);
+
+	$n <= $lower and return undef;
+
+	if (defined $upper and $upper < $n)
+	{
+	    $i += $upper - $lower;
+	    next;
+	}
+
+	return $i + $n - $lower - 1;
+    }
+
+    undef
+}
+
+
+
 sub slice
 {
     my($set, $from, $to) = @_;
@@ -1636,6 +1643,7 @@ Set::IntSpan - Manages sets of integers
 
   $n       = $set->at($i);
   $slice   = $set->slice($from, $to);
+  $i       = $set->ord($n);
 
 =head2 Operator overloads
 
@@ -2302,6 +2310,14 @@ I<$from> is negative and I<$set> is C<pos_inf>
 
 =back
 
+=item I<$i> = I<$set>->C<ord>($n)
+
+The inverse of C<at>.
+
+Returns the index I<$i> of the integer I<$n> in I<$set>,
+or C<undef> if I<$n> if not an element of I<$set>.
+
+Dies if I<$set> is C<neg_inf>.
 
 =back
 
@@ -2497,6 +2513,10 @@ Any method (except C<valid>) will C<die> if it is passed an invalid run list.
 
 (F) C<slice> was called with I<$from> negative on a positive infinite set.
 
+=item C<Set::IntSpan::ord: negative infinite set>
+
+(F) C<ord> was called on a negative infinite set.
+
 =item Out of memory!
 
 (X) C<elements> I<$set> can generate an "Out of memory!"
@@ -2617,6 +2637,10 @@ Malcolm Cook <mec@stowers-institute.org>
 
 =item *
 
+David Hawthorne <dsrthorne@hotmail.com>
+
+=item *
+
 Martin Krzywinski <martink@bcgsc.ca>
 
 =item *
@@ -2628,7 +2652,7 @@ Marc Lehmann <schmorp@schmorp.de>
 
 =head1 COPYRIGHT
 
-Copyright (c) 1996-2007 by Steven McDougall. This module is free
+Copyright (c) 1996-2010 by Steven McDougall. This module is free
 software; you can redistribute it and/or modify it under the same
 terms as Perl itself.
 
